# 
# Lib includes
#
import pyglet
import pyglet.gl
from pyglet import clock
from pyglet.window import key

from cocos.director import director
from cocos.tiles import ScrollableLayer
from cocos.layer import Layer
from cocos.scene import Scene

# Sprites

from elements.locals import *

# 
# Game includes
#
from config import *
from item import *
from player import Player, gPlayers
import resources
from rope import Rope
from globals import *
from hud import HudLayer


import logging
log = logging.getLogger('GameLayer')


class ImageLayer(Layer):
    def __init__(self, name, position, scale = 1.0):
        super( ImageLayer, self ).__init__()
        
        self.position = (position[0]*scale, position[1]*scale)
        self.img = pyglet.resource.image(name)
        #self.img.position = position

        
    def draw(self):
        pyglet.gl.glPushMatrix()
        self.transform()
        self.img.blit(0,0)
        pyglet.gl.glPopMatrix()




class GameLayer(ScrollableLayer):
    is_event_handler = False
    def __init__(self, scroller, phys):
        super( GameLayer, self ).__init__()
        self.phys   = phys
        self.scroller   = scroller
        self.ropeLayer = Layer()
        self.add(self.ropeLayer)

    	self.playingDeath = False

        world = self.phys.world
        plrScale = 0.5
        
        blueBank = ConvertToAnimDictionary(config.items("AnimsPlayerBlue"))
        self.playerBlue = Player(scroller, world, blueBank, True, scale = plrScale)
        self.playerBlue.schedule(self.playerBlue.update)
        gPlayers.append(self.playerBlue)
        self.add( self.playerBlue  )
       
        redBank = ConvertToAnimDictionary(config.items("AnimsPlayerRed"))

        self.playerRed = Player(scroller, world, redBank, False, scale = plrScale)
        self.playerRed.schedule(self.playerRed.update)
        gPlayers.append(self.playerRed)
        self.add( self.playerRed  )
        
        #self.phys.AddColListener(self.playerRed.body)
        
        self.scroller.set_focus(512,384)  
       
        
        self.HUD = None
        self.door = {}    
        
        
            
        
        
        
    def Load(self):
        log.info("Load")
        self.door = {}
        
        world = self.phys.world

        items = config.items("Items")
        for itemEntry in items:
            location = eval(itemEntry[1])          
            location = (location[0], SCREEN_SIZEY - location[1]) 
            itemname = itemEntry[0].strip('0123456789').upper()

            isDynamic = True
            if (config.has_section("ItemSettings"+itemname)):
                isDynamic = config.get("ItemSettings"+itemname, "Static") == "False"                
            
            if (itemname.startswith("DOOR_")):
                doorBank = ConvertToAnimDictionary([
                                                    ("Opening", config.get("ItemSettings"+itemname,"Opening")), 
                                                    ("Opened", config.get("ItemSettings"+itemname,"Opened"))
                                                    ])
                doorBank["CLOSED"] = gItemTextures[itemname]
                item = Door(itemname, world,  doorBank, position = location, scale = 0.5)
                self.door[itemname] = item
            else:
                item = SquareItem(itemname, world, gItemTextures[itemname] , position = location, isdynamic = isDynamic)
                
            gItems.append(item)
            self.add( item  )

        self.ResetLevel()
        
        self.rope = Rope(world, self.ropeLayer, self.playerBlue, self.playerRed, int(config.get("Player","RopeLinks")))  
        
        #world.add.Rope(self.playerBlue.body, self.playerRed.body, 30)      
        scale = 1.0
        
        offset = (0,0)
        
        if config.get("LevelSettings","Scroll")== "True":
            offset = (0,-768)
        #    log.info("Level scrolling on")
        #    world.camera.track(self.playerBlue.body)
        #    #offset = (0,0)#(512,384)
                     

        else:
        #    log.info("Level scrolling off")
            world.camera.center(offset)
            offset = (0,0)
        #world.camera.center(offset)
        bg = ImageLayer(config.get("Display","background"), offset, scale)
        self.add( Scene (bg), z=-3)
        self.add( Scene (ImageLayer(config.get("Display","foreground"), offset, scale)), z=-2 )
        
        #self.phys.AddGround((-bg.img.width,-bg.img.height/2),(bg.img.width,-bg.img.height/2))
        self.phys.AddGround((-bg.img.width*4,768-bg.img.height),(bg.img.width*4,768-bg.img.height))

    def StartGame(self):
        gAudio[0].Init()
	gAudio[0].LoadSfx('sfx_death.ogg')
	gAudio[0].LoadSfx('test.ogg')
        gAudio[0].PlayMusic("dirty_puddles_loop.ogg", "dirty_puddles_intro.ogg")
        clock.schedule(self.phys.UpdateWorld)
        clock.schedule(self.UpdateScroll)
        
    def UpdateScroll(self, dt =0):
        log.debug("UpdateScroll")
        """ 
        pos = self.playerBlue.body.GetPosition()
        pos = [pos.x*gPPM, pos.y*gPPM]
        log.debug("Pos:"+str(pos))
        
        pos = self.phys.world.to_world(pos)
        log.debug("Pos:"+str(pos))

        self.scroller.set_focus( pos[0], pos[1])
        """
        #self.scroller.set_focus(self.playerBlue.x, self.playerBlue.y)
        if config.get("LevelSettings","Scroll")== "True":
            log.debug("parent:"+str((self.parent.x, self.parent.y)))
            log.debug("self:"+str((self.x, self.y)))
            log.debug("playerBlue:"+str((self.playerBlue.x, self.playerBlue.y)))
            x = self.playerBlue.x-512
            y = self.playerBlue.y-384
            
            w = int(config.get("LevelSettings","Width")) - 1024
            h = int(config.get("LevelSettings","Height")) - 768
            
            if (x<0):
                x = 0
            if (y>0):
                y = 0
            if (y<-h):
                y = -h
            if (x>w):
                x = w
            self.set_view(x,y, 1024, 768)

        
    def PlayerFellOffWorld(self):
	print "PlayerFellOffWorld"
        log.info("PlayerFellOffWorld")
	if not self.playingDeath:
        	self.PlayDeathNoise()

    def PlayDeathNoise(self):
	print "Playing death noise"
	gAudio[0].Init()
        gAudio[0].PlayMusic('sfx_death.ogg')
	clock.schedule_interval(self.WaitForDeathSfx, 5)
	self.playingDeath = True
	
    def WaitForDeathSfx(self, useless):
	print "Wait for death sfx called"
	if (self.parent.needRebuild == False ):
	    self.parent.needRebuild = True
	    self.parent.won = False 
	    self.HUD.RemoveLife()
            self.playingDeath = False

        log.info("PlayerFellOffWorld complete")
	clock.unschedule(self.WaitForDeathSfx)
		
        
    def PlayerWin(self):
        log.info("PlayerWin")
        if (self.parent.needRebuild == False ):
            self.parent.needRebuild = True
            self.parent.won = True    
            self.parent.nextLevelScene.level += 1
            clock.unschedule(self.phys.UpdateWorld)
            clock.unschedule(self.UpdateScroll)
            
        log.info("PlayerWin complete")
        
    def UnlockDoor(self, door):
        log.info("UnlockDoor:"+door)
        self.door[door].Unlock()
        
    def ResetDoors(self):
        log.info("ResetDoors")
        for door in self.door.values():
            door.Reset()           
        
    def ResetLevel(self):
        # 

        
        log.info("ResetLevel")
        
        # Clear player inventory
        self.HUD.RemoveAllInventory()
        
        # Reset players
        def setPos(plr,pos):
            plr.position = pos
            plr.body.SetXForm(box2d.b2Vec2(pos[0]/gPPM , (SCREEN_SIZEY-pos[1])/gPPM ), 0.0) 
            
        setPos(self.playerBlue, eval(config.get("Player","StartPosBlue")))
        setPos(self.playerRed, eval(config.get("Player","StartPosRed")))
        
        # Reset Objects
        self.ResetDoors()
        
        
        
    def on_key_press (self, keys, modifiers):
        for player in gPlayers:
            player.KeyPress(keys)
            
        if (keys == key.R and self.parent.needRebuild == False ):
            log.info("Respawn requested.")
            self.PlayerFellOffWorld()
            return
            
        if (keys == key.F1):
            log.info("Leave game.")
            clock.unschedule(self.phys.UpdateWorld)
            clock.unschedule(self.UpdateScroll)
            self.parent.parent.switch_to(0)
            
            

    def on_key_release (self, key, modifiers):
        for player in gPlayers:
            player.KeyRelease(key)

    def on_enter(self):
        super( GameLayer, self).on_enter()
        self.is_event_handler = True
        self.StartGame()
        
    def on_exit(self):
        super( GameLayer, self).on_exit()
        self.is_event_handler = False        
        clock.unschedule(self.phys.UpdateWorld)
        clock.unschedule(self.UpdateScroll)
 

