# 
# Lib includes
#
import pyglet
import pyglet.gl
from pyglet import clock

from cocos.director import director
from cocos.tiles import ScrollableLayer
from cocos.layer import Layer
from cocos.scene import Scene
from cocos.text import Label
from cocos.actions import *

# 
# Game includes
#
from config import *
from item import *
from player import Player, gPlayers
import resources
from rope import Rope
from globals import *

import logging
log = logging.getLogger('HUD')


class HudLayer(Layer):
    def __init__(self):
        super( HudLayer, self ).__init__()
        
        self.img = pyglet.resource.image("hud.png")
        self.inventory = {}
        self.lives = 3
        self.lifeText = None
        self._UpdateHUD()

    def draw(self):
        pyglet.gl.glPushMatrix()
        self.transform()
        self.img.blit(0,0)
        pyglet.gl.glPopMatrix()
        super( HudLayer, self).draw()

    def AddtoInventory(self, item):
        self.inventory[item] = Sprite(gItemTextures[item], position = (50,80))
        self.add(self.inventory[item])
        #scale = ScaleBy(0.9, duration=4)
        #self.inventory[item].do( Repeat( scale + Reverse(scale) ) )
        log.info( "AddtoInventory:"+ item)
        
    def RemoveFromInventory(self, item):
        self.remove(self.inventory[item])
        del self.inventory[item]
        log.info( "RemoveFromInventory:"+ item)
        
    def RemoveAllInventory(self):
        for item in self.inventory.values():
            self.remove(item)
        self.inventory = {}
        log.info( "RemoveAllInventory")
   
    def Reset(self):
        self.lives = 3
        self._UpdateHUD()
        
    def AddLife(self):
        log.info( "AddLife")
        self.lives += 1
        self._UpdateHUD()
    def RemoveLife(self):
        log.info( "RemoveLife")
        self.lives -= 1
        self._UpdateHUD()
    def _UpdateHUD(self):
        if (self.lifeText):
            self.remove(self.lifeText) 

        self.lifeText = Label(str(self.lives), font_name='Times New Roman', font_size=22, anchor_x='center', anchor_y='center', color = (0,0,0,255))

        # set the label in the center of the screen
        self.lifeText.position = 140,40
        self.add( self.lifeText )
        
        #scale = ScaleBy(2, duration=5)
        #self.lifeText.do( Repeat( scale + Reverse( scale) ) )

    def on_enter(self):
        super( HudLayer, self).on_enter()
        
    def on_exit(self):
        super( HudLayer, self).on_exit()
 

