from cocos.sprite import *
from main import box2d
import elements 
import math
import pyglet
from pyglet.image import Animation

from globals import *
import logging
log = logging.getLogger('Items')

# 
# Game includes
#
from config import *

gItems = []

def ConvertToAnimDictionary(tupleList):
    dict = {}
    for k,v in tupleList:
        dict[k.upper()] = pyglet.resource.animation(v)
        
    return dict

gItemTextures = ConvertToAnimDictionary(config.items("ItemData"))

class GameItem(object):
    def __init__(self, type):
        self.itemType = type
        self.collisionQueue = []
        self.taggedForRemoval = False
        self.mCollisionList = []

    def update(self, dt):
        pos = self.body.GetPosition()
        pos = [pos.x*gPPM,pos.y*gPPM]
        angle = (self.body.GetAngle()*-180.0)/math.pi
        self.position = pos
        self.rotation = angle
        self.GetCollisions()

    def Collide(self, item):
        log.debug("GameItem::Collide")
        print "GameItem::Collide: ("+self.itemType+")",item.itemType

    def AddCollision(self, c):
        log.debug("GameItem::AddCollision")
        log.debug(self.itemType+"-"+c.itemType)
        self.collisionQueue.append(c)

    def GetCollisions(self):
        #log.debug("GameItem::GetCollisions")
        while self.collisionQueue:
            col = self.collisionQueue.pop()
            self.Collide(col)

    def Static(self, obj):
        pass
    def Hold(self, obj):
        pass
    def Pickup(self, obj):
        pass
    def Consume(self, obj):
        pass
    def Autoconsume(self, obj):
        pass
    def Inventory(self, obj):
        pass


class SquareItem(Sprite,GameItem):
    def __init__(self, type, world, image, position=(0,0), rotation=0, scale=1, opacity = 255, color=(255,255,255), anchor = None, 
                 isfixedRotation = False, width = None,height = None, isdynamic = True, usefg = None, density=3.8, restitution=0.16, friction=0.85):

        Sprite.__init__(self, image, position, rotation, scale, opacity, color, anchor)
        GameItem.__init__(self, type)
        
        if (width):
            self.image.width = width
        if (height):
            self.image.width = height
   
        self.world = world
        
        width = 0
        height = 0
        
        if isinstance(self.image, Animation):
            width = image.frames[0].image.width
            height = image.frames[0].image.height
        else:
            width = image.width
            height = image.height
        
        self.body = self.world.add.rect(position, scale*(width/2.0), scale*(height/2.0), 
                                        density=density, restitution=restitution, friction=friction, fixedRotation = isfixedRotation, dynamic = isdynamic, fg=usefg)

        #self.body.SetUserData(self)
        # This blows up debug drawing
        self.schedule(self.update)
        
    def Collide(self, item):
        log.debug("SquareItem::Collide")
        GameItem.Collide(self, item)
        
    def AddCollision(self, c):
        log.debug("SquareItem::AddCollision")
        GameItem.AddCollision(self, c)


class RoundItem(Sprite,GameItem):
    def __init__(self, type, world, image, position=(0,0), rotation=0, scale=1, opacity = 255, color=(255,255,255), anchor = None):

        Sprite.__init__(self, image, position, rotation, scale, opacity, color, anchor)
        GameItem.__init__(self, type)
        
        self.world = world
        
        width = 0
        
        if isinstance(self.image, Animation):
            width = image.frames[0].image.width
        else:
            width = image.width
        
        self.body = self.world.add.ball(position, scale*(width/2.0))
        #self.body.SetUserData(self)
        # This blows up debug drawing
        self.schedule(self.update)
        
    def Collide(self, item):
        log.debug("RoundItem::Collide")
        GameItem.Collide(self, item)
        
    def AddCollision(self, c):
        log.debug("RoundItem::AddCollision")
        GameItem.AddCollision(self, c)
 
class Door(SquareItem):
    def __init__(self, name, world, imageBank, position=(0,0), rotation=0, scale=1, opacity = 255, color=(255,255,255), isdynamic = False, anchor = None):
        self.imageBank = imageBank
        super( Door, self ).__init__(name, world, self.imageBank["CLOSED"], position, rotation, scale, opacity, color, anchor,isdynamic = False)
        self.locked = True
        self.open = False

        log.debug("Door::__init__ "+str(position))

    def Unlock(self):
        log.debug("Door::Unlock")
        self.locked = False
        self.SetAnim("OPENING")
        
    def Reset(self):
        log.debug("Door::Reset")
        self.locked = True
        self.open = False
        self.SetAnim("CLOSED")
        
    def SetAnim(self, anim):
        log.debug("Door::SetAnim")
        self.image = self.imageBank[anim]
        self.image_anchor = (self.image.get_max_width()/2, self.image.get_max_height()/2)

    def on_animation_end(self):
        #log.debug("Door::on_animation_end")
        if (self.image == self.imageBank["OPENED"]):
            self.open = True
             
        if (self.image == self.imageBank["OPENING"]):
            self.SetAnim("OPENED") 
               
    def Collide(self, item):
        log.debug("Door::Collide")
        GameItem.Collide(self, item)
        
    def AddCollision(self, c):
        log.debug("Door::AddCollision")
        GameItem.AddCollision(self, c)

    def Static(self):
        log.debug("Door::Staric")


    
