'''

'''
#
# Python includes
#
import os, sys
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..'))

import logging
import math

#
# Library includes
#

# Pygame
import pygame
from pygame.locals import *

# Sprites

# Physics
import Box2D2 as box2d

# Window and event handling
from pyglet.window import Window
from pyglet import clock, image, window
from pyglet.window import *
import pyglet

from cocos.director import director
from cocos.layer import *
from cocos.scene import Scene
from cocos.scenes.transitions import *
from cocos.actions import *
from cocos.sprite import *
from cocos.menu import *
from cocos.text import *
from cocos.tiles import ScrollingManager


# 
# Game includes
#
from config import *

import resources

resources.GetResources()

import menus

from item import SquareItem, RoundItem
from player import Player

from physicsworld import PhysicsWorld
from gamelayer import GameLayer, ImageLayer
from level import LevelLayer
from intro import IntroLayer

from audio import Audio
from globals import *
from hud import HudLayer

SCREEN_SIZE = [int(config.get("ScreenSize", "width")), int(config.get("ScreenSize", "height"))]

# set up logging to file - see previous section for more details
logging.basicConfig(level = eval(config.get("Debug","logLevel")),
                    format   ='%(asctime)s %(name)-12s %(levelname)-8s %(message)s',
                    datefmt  ='%H:%M:%S',
                    filename = config.get("Debug","fileName"),
                    filemode = 'w')
                    
# define a Handler which writes INFO messages or higher to the sys.stderr
console = logging.StreamHandler()
console.setLevel(eval(config.get("Debug","consoleLogLevel")))
# set a format which is simpler for console use
formatter = logging.Formatter('%(name)-12s: %(levelname)-8s %(message)s')
# tell the handler to use this format
console.setFormatter(formatter)
# add the handler to the root logger
logging.getLogger('').addHandler(console)

log = logging.getLogger('main')
log.info('Starting main...')
    
current_scene = 0
intro_scenes = []

class SwitchLayer(Scene):
    def __init__(self, scroller):
        super(SwitchLayer, self).__init__(scroller)
        
        self.physics = None
        self.gameScene = None
        self.level = None
        self.scroller = scroller
        self.nextLevelScene = None
         
        self.needRebuild = False
        self.won = False    

        self.HUD = HudLayer()
        self.add(self.HUD )
        self.RebuildScene()
        clock.schedule(self.update)

    def RebuildScene(self):
        log.debug("SwitchLayer::RebuildScene")
        
        self.needRebuild = False
        
        if (self.HUD.lives <= 0):
            log.debug("SwitchLayer::RebuildScene lives <= 0")
            self.HUD.Reset()
            self.won = False 
            self.parent.switch_to(4)
            return
            
        if (self.won):
            log.debug("SwitchLayer::RebuildScene won == True")
            self.won = False    
            self.parent.switch_to(6)
            

        log.debug("SwitchLayer::RebuildScene Rebuilding Physics")
        
        if (self.physics):
            self.remove(self.physics)
            self.physics = None
        self.physics = PhysicsWorld(SCREEN_SIZE)
        self.add(self.physics)
        
        log.debug("SwitchLayer::RebuildScene Rebuilding Game Scene")
        
        if (self.gameScene):
            self.remove(self.gameScene)
            self.gameScene = None
        self.gameScene = GameLayer (self.scroller, self.physics)
        self.add(self.gameScene)
        
        log.debug("SwitchLayer::RebuildScene Rebuilding level")
        
        if (self.level):
            log.debug("SwitchLayer::RebuildScene Removing old level")
            self.remove(self.level)
            log.debug("SwitchLayer::RebuildScene Deleting old level")
            self.level = None
            
        log.debug("SwitchLayer::RebuildScene Getting ready to load")
        
        self.level = LevelLayer(self.physics.world, SCREEN_SIZE[1])
        
        log.debug("SwitchLayer::RebuildScene adding new level")
        
        self.add(self.level)
        
        log.debug("SwitchLayer::RebuildScene Getting ready to load")
        
        self.gameScene.HUD = self.HUD
        
        log.debug("SwitchLayer::RebuildScene Loading")
        
        self.gameScene.Load()
        
        log.debug("SwitchLayer::RebuildScene complete")
   
       
    def on_enter(self):
        super(SwitchLayer, self).on_enter()
    #    self.do(FadeOut(3)+Delay(2.5)+FadeIn(3))

    def update(self,dt):
        if self.needRebuild:
            log.debug("SwitchLayer::update Need Rebuild")
            self.needRebuild = False
            self.on_enter()
            self.RebuildScene()
            
class GameOverScene(Scene):
    def __init__(self):
        super(GameOverScene, self).__init__()
        
    def on_enter(self):
        log.debug("GameOverScene::on_enter")
        super(GameOverScene, self).on_enter()
        gAudio[0].PlayMusic("sfx_death2.ogg", "gap.ogg")
        clock.schedule_once(self.GoBack, 5)

    def GoBack(self, dt):
        log.debug("GameOverScene::GoBack")
        self.parent.switch_to(0)
        
class LevelCompleteScene(Scene):
    def __init__(self):
        super(LevelCompleteScene, self).__init__()
        
    def on_enter(self):
        log.debug("LevelCompleteScene::on_enter")
        super(LevelCompleteScene, self).on_enter()
        #gAudio[0].PlayMusic("rain_catcher.ogg", "gap.ogg")
        clock.schedule_once(self.GoBack, 5)

    def GoBack(self, dt):
        log.debug("LevelCompleteScene::GoBack")
        self.parent.switch_to(3)

class NextLevelScene(Scene):
    def __init__(self, levels):
        super(NextLevelScene, self).__init__()
        self.levels = levels
        self.level = 0
        self.current = None
        self.switchto = 3
        
    def on_enter(self):
        super(NextLevelScene, self).on_enter()
        log.debug("NextLevelScene::on_enter")
        if (self.current):
            log.debug("NextLevelScene::on_enter Removing current")
            self.remove(self.current)
        
        if (self.level < len(self.levels)):
            log.debug("NextLevelScene::on_enter We havent finished")
            self.current = self.levels[self.level]   
            log.debug("NextLevelScene::on_enter Level:"+str(self.level))
            self.add(self.current)            
        else:
            self.switchto = 8

        clock.schedule_once(self.GoBack, 5)

    def GoBack(self, dt):
        log.debug("NextLevelScene::GoBack")
        # Level data
        config.read("data/levels/level"+str(self.level)+".ini")
        
        self.parent.switch_to( self.switchto)
        
class CreditsScene(Scene):
    def __init__(self, backto, img):
        super(CreditsScene, self).__init__(img)
        self.backto = backto
        
    def on_enter(self):
        log.debug("CreditsScene::on_enter")
        super(CreditsScene, self).on_enter()
        #gAudio[0].PlayMusic("rain_catcher.ogg", "gap.ogg")
        clock.schedule_once(self.GoBack, 8)

    def GoBack(self, dt):
        log.debug("CreditsScene::GoBack")
        self.parent.switch_to(self.backto)

def run():
    
    director.init(width=SCREEN_SIZE[0], height=SCREEN_SIZE[1])

    director.window.set_caption("NaN - Pyweek7 - The day mom tied us together")

    #icon = image.load("data/icon.ico")
    #director.window.set_icon(icon)

    bg_layer = ImageLayer("mainmenu.png", (0, 0), 1.0)
    main_scene = Scene (ColorLayer(255,255,255,255), bg_layer)   

    scroller = ScrollingManager()
    scroller_scene = SwitchLayer (scroller)

    failScene = CreditsScene(0, ImageLayer("gameoverscreen.png", (0, 0), 1.0))
    
    levelCompleteLayer = ImageLayer("levelcompletescreen.png", (0, 0), 1.0)
    levelCompleteScene = LevelCompleteScene()
    levelCompleteScene.add(levelCompleteLayer)
    
    levelCompleteLayers = [
                           ImageLayer("level1screen.png", (0, 0), 1.0),
                           ImageLayer("level2screen.png", (0, 0), 1.0),
                           ImageLayer("level3screen.png", (0, 0), 1.0)
                           ]
    nextLevelScene = NextLevelScene(levelCompleteLayers)
    scroller_scene.nextLevelScene = nextLevelScene

    gAudio.append(Audio())
    credits = CreditsScene(0, ImageLayer("credits.png", (0, 0), 1.0))
    youwin = CreditsScene(7, ImageLayer("youwin.png", (0, 0), 1.0))
    instructions = CreditsScene(6, ImageLayer("instructionscreen.png", (0, 0), 1.0))


    layers = MultiplexLayer(
                        menus.MainMenu(),       # 0
                        menus.OptionsMenu(),    # 1
                        menus.HelpMenu(),       # 2
                        scroller_scene,         # 3
                        failScene,              # 4
                        levelCompleteScene,     # 5
                        nextLevelScene,         # 6
                        credits,                # 7
                        youwin,                 # 8
                        instructions,           # 9
                         ) 
    
    if (config.get("Debug","SkipToGame") == "True"):
        layers.layers[3].gameScene.is_event_handler = True
        layers.layers[3].enable_handlers(True)
        layers.switch_to(3)        
    elif (config.get("Debug","SkipToMenu") == "True"):
        layers.switch_to(0)
    else:
        
        intro_scenes.append(Scene(IntroLayer("nanlogo.png", 4)))
        intro_scenes.append(Scene(IntroLayer("1.jpg",4 )))
        intro_scenes.append(Scene(IntroLayer("2.jpg",12 )))
        intro_scenes.append(Scene(IntroLayer("3.jpg",11 )))
        intro_scenes.append(Scene(IntroLayer("4.jpg",8 )))
        intro_scenes.append(Scene(IntroLayer("5.jpg",13 )))
        intro_scenes.append(Scene(IntroLayer("6.jpg",11 )))
        intro_scenes.append(Scene(IntroLayer("7.jpg",11 )))
        intro_scenes.append(Scene(IntroLayer("8.jpg",6 )))
        intro_scenes.append(Scene(IntroLayer("9.jpg",9 )))
    
        # add the intro scenes in order, decreasing z-order as we go
        z = 50
        for i in intro_scenes:
            main_scene.add(i, z)
            z -= 1
            
        current_scene = 0
        intro_scenes[current_scene].schedule_interval(next_intro, 4)
        
    main_scene.add(layers, z=3 )
    director.run (main_scene)

# for the intro sequence
def next_intro(useless):
	global current_scene
	global intro_scenes
	global gAudio

	intro_scenes[current_scene].unschedule(next_intro)
	intro_scenes[current_scene].do(Hide())
	current_scene += 1

	if current_scene == 2:
    		gAudio[0].PlayMusic("nan_theme.ogg", "gap.ogg")
	try:
		children = intro_scenes[current_scene].children
		z, ref = children[0]
    		intro_scenes[current_scene].schedule_interval(next_intro, ref.getDelay())
	except:
		pass

if __name__ == "__main__":
    run()



 
