'''

'''
#
# Python includes
#
import os, sys
import logging

#
# Library includes
#

# Pygame
import pygame
from pygame.locals import *

# Sprites
import rabbyt

# Window and event handling
from pyglet.window import Window
from pyglet import clock, image, window
from pyglet.gl import *
from pyglet.window import *
import pyglet
from pyglet import font

# Physics
import Box2D2 as box2d

# 
# Game includes
#
from config import *

SCREEN_SIZE = [int(config.get("ScreenSize", "width")), int(config.get("ScreenSize", "height"))]

rabbyt.data_directory = os.getcwd()+"\\data"

clock.schedule(rabbyt.add_time)

pygame.init()
pygame.mixer.init()

window = Window(width=SCREEN_SIZE[0], height=SCREEN_SIZE[1])
rabbyt.set_default_attribs()

keys = key.KeyStateHandler()

window.push_handlers(keys)
#window.push_handlers(pyglet.window.event.WindowEventLogger())
window.set_caption("Team NaN - Pyweek7 - Test")
icon = image.load("data/icon.ico")
window.set_icon(icon)
arial = font.load('Arial', 10, bold=True, italic=False)
fps = clock.ClockDisplay(font = arial, color=(1, .5, .5, 1))

window.push_handlers(fps)


# set up logging to file - see previous section for more details
logging.basicConfig(level = eval(config.get("Debug","logLevel")),
                    format   ='%(asctime)s %(name)-12s %(levelname)-8s %(message)s',
                    datefmt  ='%m-%d %H:%M:%S',
                    filename = config.get("Debug","fileName"),
                    filemode = 'w')
                    
# define a Handler which writes INFO messages or higher to the sys.stderr
console = logging.StreamHandler()
console.setLevel(eval(config.get("Debug","consoleLogLevel")))
# set a format which is simpler for console use
formatter = logging.Formatter('%(name)-12s: %(levelname)-8s %(message)s')
# tell the handler to use this format
console.setFormatter(formatter)
# add the handler to the root logger
logging.getLogger('').addHandler(console)


# Test the sprites
test = rabbyt.Sprite("nanlogo.png")
test.x = 400
test.y = 300
test.alpha = 0.3

# Test the sounds
testsound = pygame.mixer.Sound("data\\audio\\test.ogg")

"""
# Test physics using box2d
# World
worldAABB = box2d.b2AABB()
worldAABB.lowerBound.Set(-10.0, -10.0)
worldAABB.upperBound.Set( SCREEN_SIZE[0]+10,SCREEN_SIZE[1]+10)

# Set gravity
gravity = box2d.b2Vec2(0.0, -10.0)

doSleep = True
world = box2d.b2World(worldAABB, gravity, doSleep)

world.SetGravity(box2d.b2Vec2(0.0, 0.0))


bd=box2d.b2BodyDef() 
bd.position.Set(0.0, 20.0)
ground = world.CreateBody(bd) 

sd=box2d.b2PolygonDef() 
sd.density = 0.0
sd.restitution = 0.4

# Top
sd.SetAsBox(SCREEN_SIZE[0]-20, 20.0, box2d.b2Vec2(SCREEN_SIZE[0]/2, SCREEN_SIZE[1]-10), 180.0)
ground.CreateShape(sd)
# Bottom
sd.SetAsBox(SCREEN_SIZE[0]-20, 20.0, box2d.b2Vec2(SCREEN_SIZE[0]/2, 10.0), 180.0)
ground.CreateShape(sd)
# Left
sd.SetAsBox(20.0, SCREEN_SIZE[1]-20, box2d.b2Vec2(10.0, SCREEN_SIZE[1]/2), 0.0)
ground.CreateShape(sd)
# Right
sd.SetAsBox(20.0, SCREEN_SIZE[1]-20, box2d.b2Vec2(SCREEN_SIZE[0]-10, SCREEN_SIZE[1]/2), 0.0)
ground.CreateShape(sd)

class TestGuy():
	log = logging.getLogger('TestGuy')
	def __init__(self):
		self.log.debug('__init__') 
		self.image = rabbyt.Sprite("icon.ico")
		
		self.bodyDef = box2d.b2BodyDef()
		self.bodyDef.allowSleep = True
		self.bodyDef.position.Set(box2d.b2Random(40,SCREEN_SIZE[0]-40), box2d.b2Random(40,SCREEN_SIZE[1]-40))
		#self.bodyDef.isBullet = True
		
		self.body = world.CreateBody(self.bodyDef)
		self.body.SetLinearVelocity(-5.0 * self.bodyDef.position)

		#obj = box2d.b2CircleDef()
		#obj.radius = 10
		obj = box2d.b2PolygonDef() 

		obj.SetAsBox(-self.image.left+self.image.right, -self.image.bottom+self.image.top)

		obj.density = 1.0
		obj.restitution = 1.0
		obj.friction = 0.1

		
		self.body.CreateShape(obj)
		self.body.SetMassFromShapes()
		
		
	def update(self):
		self.log.debug('update') 
		pos = self.body.GetPosition()
		angle = self.body.GetAngle()
		self.image.xy = [pos.x,pos.y]
		self.image.angle = angle
		
	def render(self):
		self.log.debug('render') 
		self.image.render()

# Create a bunch of objects
testGuys = []
for i in range(int(config.get("Test","numObjects"))):
	testGuys.append(TestGuy())
"""

# Test physics using the elements wrapper for box2d
try:
    import elements                 
except ImportError:
    print "Elements lib not found"
    
      

from math import pi

world = elements.Elements(SCREEN_SIZE, renderer='opengl_pyglet')
world.set_inputAxisOrigin(left=True, top=False)
world.add.ground()
class TestGuy():
    log = logging.getLogger('TestGuy')
    def __init__(self):
        self.log.debug('__init__') 
        self.image = rabbyt.Sprite("nanlogo.png")
        self.image.shape = [-32, -32, 32, 32]
        
        size = (-self.image.left+self.image.right, -self.image.bottom+self.image.top)
        self.body = world.add.rect((400,400 ), size[0]/2, size[1]/2)
        
    def update(self):
        self.log.debug('update') 
        pos = self.body.GetPosition()
        pos = [pos.x*100.0,pos.y*100.0]
        angle = (self.body.GetAngle()*180.0)/pi

        self.image.xy = pos
        self.image.rot = angle
        
        
    def render(self):
        self.log.debug('render') 
        self.image.render()


testGuys = []

for i in range(int(config.get("Test","numObjects"))):
    testGuys.append(TestGuy())

# The main shipoopie
def run():
    logging.info('Starting main...')
    
    log = logging.getLogger('main')
    
    # Test ogg sound
    testsound.play(15, 1000, 200)

    while not window.has_exit:
        """ 
	    world.Step(1.0/60.0, 10)
        world.Validate()
        """    
    	for testGuy in testGuys:
	    	testGuy.update()

    	world.update()
    	rabbyt.clear(rgba=(0.0,0.0,0.0,1.0))

        clock.tick(40)
        rabbyt.scheduler.pump()
        test.render()
        
        glDisable(GL_TEXTURE_2D)
        world.draw()
        glEnable(GL_TEXTURE_2D)
        
        # Draw the GUI
        #window.draw()
        fps.draw()  
        
        
    	for testGuy in testGuys:
        	testGuy.render()

        # Do these last
        window.dispatch_events() 
        window.flip()
        
if __name__ == "__main__":
	run()
