# This code is so you can run the samples without installing the package
import sys
import os
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..'))

from operator import setslice

from cocos.director import director
from cocos.layer import *
from cocos.scene import Scene
from cocos.scenes.transitions import *
from cocos.actions import *
from cocos.sprite import *
from cocos.menu import *
from cocos.text import *

import pyglet
from pyglet import gl, font
from pyglet.window import key


# 
# Game includes
#
from config import *

from globals import gAudio

MENU_TITLE = "NAN 7 Test"

TITLE_FONT = config.get("Menu","TitleFont")
MENU_FONT = config.get("Menu","MenuItemFont")
MENU_FONT_SIZE = 20

class OptionsMenu( Menu ):
    def __init__(self):
        super( OptionsMenu, self).__init__(":Options:") 
        
        self.select_sound = gAudio[0].LoadSfx('test.ogg')

        self.font_title['font_name'] = TITLE_FONT
        self.font_title['font_size'] = MENU_FONT_SIZE
        self.font_title['color'] = (204,10,10,255)

        self.font_item['font_name'] = MENU_FONT
        self.font_item['color'] = (0,0,0,255)
        self.font_item['font_size'] = MENU_FONT_SIZE
        self.font_item_selected['font_name'] = MENU_FONT
        self.font_item_selected['color'] = (0,0,0,255)
        self.font_item_selected['font_size'] = MENU_FONT_SIZE

        self.menu_valign = BOTTOM
        self.menu_halign = CENTER
        
        self.menu_anchor_y = CENTER
        self.menu_anchor_x = CENTER

        items = []

        #self.levels = ['0','1','2']
        self.volumes = ['Mute','10','20','30','40','50','60','70','80','90','100']
        
        musicSelected = int(self.volumes.index(config.get("Audio","musicVolume")))
        self.on_music_volume(musicSelected)        
        items.append( MultipleMenuItem(
                        'Music volume: ', 
                        self.on_music_volume,
                        self.volumes,
                        musicSelected)
                    )
        
        sfxSelected =  int(self.volumes.index(config.get("Audio","sfxVolume")))
        self.on_sfx_volume(sfxSelected)
        items.append( MultipleMenuItem(
                        'SFX volume: ', 
                        self.on_sfx_volume,
                        self.volumes,
                       sfxSelected )
                    )
        items.append( ToggleMenuItem('Show FPS:', self.on_show_fps, director.show_FPS) )
        items.append( MenuItem('Fullscreen', self.on_fullscreen) )
        items.append( MenuItem('Back', self.on_quit) )
        self.create_menu( items,  zoom_in(), zoom_out())

    def on_fullscreen( self ):
        director.window.set_fullscreen( not director.window.fullscreen )

    def on_quit( self ):
        self.parent.switch_to( 0 )

    def on_show_fps( self, value ):
        director.show_FPS = value

    def on_sfx_volume( self, idx ):
        try:
            gAudio[0].SetSfxVolume(int(self.volumes[idx]))
        except:
            gAudio[0].SetSfxVolume(0)
        
    def on_music_volume( self, idx ):
        try:
            gAudio[0].SetMusicVolume(int(self.volumes[idx]))
        except:
            gAudio[0].SetMusicVolume(0)
        
class HelpMenu( Menu ):
    def __init__(self):
        super( HelpMenu, self).__init__(":Help:") 
        
        self.select_sound = gAudio[0].LoadSfx('test.ogg')

        self.font_title['font_name'] = TITLE_FONT
        self.font_title['font_size'] = MENU_FONT_SIZE
        self.font_title['color'] = (204,10,10,255)


        self.font_item['font_name'] = MENU_FONT
        self.font_item['color'] = (0,0,0,255)
        self.font_item['font_size'] = MENU_FONT_SIZE
        self.font_item_selected['font_name'] = MENU_FONT
        self.font_item_selected['color'] = (0,0,0,255)
        self.font_item_selected['font_size'] = MENU_FONT_SIZE

        self.menu_valign = BOTTOM
        self.menu_halign = CENTER
        
        items = []

 
        items.append( MenuItem('Back', self.on_quit)  )
        self.create_menu( items, zoom_out(), zoom_in() )
        

    def on_enter(self):
        super( HelpMenu, self).on_enter()
        
        

    def on_quit( self ):
        self.parent.switch_to( 0 )



class MainMenu( Menu ):

    def __init__(self):
        super( MainMenu, self).__init__("") 

        self.select_sound = gAudio[0].LoadSfx('test.ogg')


        self.font_item['font_name'] = MENU_FONT
        self.font_item['color'] = (0,0,0,255)
        self.font_item['font_size'] = MENU_FONT_SIZE
        self.font_item_selected['font_name'] = MENU_FONT
        self.font_item_selected['color'] = (0,0,0,255)
        self.font_item_selected['font_size'] = MENU_FONT_SIZE

        self.menu_valign = BOTTOM
        self.menu_halign = CENTER

        items = []

        items.append( MenuItem('New Game', self.on_newgame) )
        #items.append( MenuItem('Instructions', self.on_help) )
        #items.append( MenuItem('Story', self.on_story) )
        items.append( MenuItem('Credits', self.on_credits) )
        #items.append( MenuItem('Options', self.on_options) )
        items.append( MenuItem('Quit', self.on_quit) )

        
        self.create_menu( items, zoom_in(), zoom_out() )
        c = 0
        for i in items:
            i.y += 150-c*10
            c +=1

    def on_options( self ):
        self.parent.switch_to(1)
          
    def on_help( self ):
        self.parent.switch_to(2)
      
    def on_story( self ):
        self.parent.switch_to(1)
        
    def on_credits( self ):
        self.parent.switch_to(7)
        
    def on_newgame( self ):
        print self.parent.layers[3].children
        #self.parent.layers[3].children[2][1].is_event_handler = True
        self.parent.layers[3].gameScene.is_event_handler = True
        self.parent.layers[3].enable_handlers(True)
        self.parent.switch_to(9)

    def on_quit(self):
        gAudio[0].PlayQuit()
        pyglet.app.exit()

if __name__ == "__main__":


    director.init( resizable=True, width=800, height=600 )
    scene = Scene()
    scene.add( MultiplexLayer(
                    MainMenu(), 
                    OptionsMenu(),
                    HelpMenu()
                    
                    ),
                z=1 ) 
    director.run( scene )
