# 
# Lib includes
#
from pyglet.gl import *
from cocos.layer import Layer
import math

# Test physics using the elements wrapper for box2d
try:
    import elements   
    from elements.locals import *
              
except ImportError:
    print "Elements lib not found"
    #from elements.locals import *
# 
# Game includes
#
from player import gPlayers
from item import gItems
from config import *
from globals import *
from main import box2d

import logging
log = logging.getLogger('Physics')

class PhysicsWorld(Layer):
    def __init__(self, worldSize):
        super( PhysicsWorld, self ).__init__()

        self.world = elements.Elements(worldSize, renderer='opengl_pyglet', ppm=gPPM, gravity=(0.0,-9.0))
        self.world.set_inputAxisOrigin(left=True, top=False)


        self.ground = None
        
        self.world.world.SetPositionCorrection(True)
        self.world.world.SetContinuousPhysics(True)

        #self.world.camera.set_scale_factor(0.1)

        self.debugDraw = config.get("Debug","ShowPhysics") == "True"

        #def TurnOnCollisions(self):
        self.world.callbacks.add(CALLBACK_CONTACT_ADD, self.AnyCollision)
        self.world.callbacks.add(CALLBACK_CONTACT_ADD, self.ContactAdd)
        self.world.callbacks.add(CALLBACK_CONTACT_PERSIST, self.ContactPersist)

    def AddGround(self, p1, p2):
        log.info("AddGround: "+str(p1)+" "+str(p2))
        self.ground = self.world.add.wall(p1, p2)
        
    """     
    def AddColListener(self, p1):
        def contact_add_ball(c):
            print "Contact with Ball"
        self.world.callbacks.add(CALLBACK_CONTACT_ADD, contact_add_ball, [p1])
    """
    def UpdateWorld(self, delta_t):
        #if config.get("LevelSettings","Scroll")== "True":
        #    log.info("Level scrolling on")
        self.world.camera.track(gPlayers[0].body)
            #offset = (0,0)#(512,384)

        for player in gPlayers:
            if player.letMeGo and player.holdJoint:
                self.world.world.DestroyJoint(player.holdJoint)
                player.holdBody.WakeUp()
                log.info("Letting gooooooo")
                player.body.WakeUp()
                player.SetAnim("PLAYER_HANG")
                player.holdJoint = None
                player.createMeAJoint = None
                player.letMeGo = False
                gAudio[0].PlaySfx('test.ogg')
                
            if player.createMeAJoint:
                pos = player.createMeAJoint.body.GetPosition()
                pos = [pos.x*gPPM,pos.y*gPPM]
                newPlayerPos = [player.position[0]+(pos[0]-player.position[0]),player.position[1]]

                player.body.SetXForm(box2d.b2Vec2(newPlayerPos[0]/100.0 , newPlayerPos[1]/100.0 ), 0)
                player.holdBody = player.createMeAJoint.body
                #player.holdJoint = self.world.add.joint([player.createMeAJoint.body, player.body], pos)
                player.holdJoint = self.world.add.joint(player.createMeAJoint.body, player.body, pos)
                log.debug("Hold complete: "+str(player.holdJoint))
                player.SetAnim("PLAYER_GRAB")
                player.createMeAJoint = None
                player.letMeGo = False
                gAudio[0].PlaySfx('test.ogg')
                
                



        # Probably somewhere else.
        for obj in gItems:
            if obj.taggedForRemoval == True:
                gItems.remove(obj)                
                obj.world.world.DestroyBody(obj.body)
                obj.opacity = 0
                gAudio[0].PlaySfx('test.ogg')

        self.world.update(fps=70.0, iterations=30)
        #self.world.camera.center(GLOBAL_VIEW_OFFSET)
        #Layer.visit(self)

    def draw(self):
        if (self.debugDraw):
            glPushMatrix()
            self.transform()
            glDisable(GL_TEXTURE_2D)
            self.world.draw()
            glEnable(GL_TEXTURE_2D)
            glPopMatrix()

    def on_key_press (self, key, modifiers):
        print "Key Pressed:", key, modifiers

    def PlrCollision(self, plr, c, collision):
        #log.debug("PlrCollision")
        log.debug("PlrCollision Type:"+str(plr.itemType))

        plr.mCollisionList.append(collision)

        for item in gItems:
            if c == item.body:
                log.debug("\tPlayer:"+ plr.itemType +" -> "+item.itemType)
                plr.AddCollision(item)
                return
        
        if (self.ground == c):
            log.debug("PlayerFellOffWorld")
            #self.world.callbacks.add(CALLBACK_CONTACT_ADD, self.AnyCollision)
            self.parent.gameScene.PlayerFellOffWorld()
            return
        elif c.IsStatic():   
            # We are touching ground - this does not mean we are standing on it
            #log.debug("\tPlayer Landed on ground-- static: "+str( c.IsStatic()))
            return
        else:
            log.debug("\tLooking through the players")
            # Look through the players   
            for otherplr in gPlayers:
                if (c == otherplr.body and plr != otherplr):
                    log.debug("\t Hit other player"+otherplr.itemType)
                    return
            
            #log.debug("\tDidnt seem to find anything."+str(c))


    def ItemCollision(self, item, c, collision):
        #log.debug("ItemCollision")
        #log.debug("ItemCollision Type:"+str(item.itemType))
        
        if (self.ground == c):
            log.debug("ItemFellOffWorld")
            return
        elif c.IsStatic():
            #log.debug("\tLanded on ground-- static: "+str( c.IsStatic()))
            return
        else:            
            # Look through the items to see if we collided with another item
            for otheritem in gItems:
                if c == otheritem.body:
                    log.debug("\tItem:"+ otheritem.itemType+" -> "+item.itemType)
                    item.AddCollision(otheritem)
                    return
        #log.debug("\tDidnt seem to find anything.")
        
    def AnyCollision(self,c):
        log.debug("AnyCollision")
        volume = math.sqrt(c.velocity.Length()) / 14
        #print "AnyCollision Vel:", volume

        c1 = c.shape1.GetBody()
        c2 = c.shape2.GetBody()

        #gameItem1 = c1.GetUserData()
        #gameItem2 = c2.GetUserData()

        # Add this later.
        collision = ((c.position.x,c.position.y),(c.velocity.x,c.velocity.y),(c.normal.x,c.normal.y))
        #gameItem1.OnCollision(collision,gameItem2)
        #gameItem2.OnCollision(collision,gameItem1)

        #log.debug("Looking through the players")
        # Look through the players   
        for plr in gPlayers:
            if (c1 == plr.body):
                self.PlrCollision(plr, c2, collision)
                return
            elif (c2 == plr.body):
                self.PlrCollision(plr, c1, collision)
                return

        #log.debug("Looking through the items")
        # Look through the items   
        for item in gItems:
            if c1 == item.body:
                self.ItemCollision(item, c2, collision)
                return
            elif c2 == item.body:
                self.ItemCollision(item, c1, collision)
                return
            
        #log.debug("Didnt seem to find anything.")
        
    def ContactAdd(self,c):
        log.debug("ContactAdd")
        volume = math.sqrt(c.velocity.Length()) / 14
        #print "ContactAdd Vel:", volume

        c1 = c.shape1.GetBody()
        c2 = c.shape2.GetBody()
        
    def ContactPersist(self,c):
        log.debug("ContactPersist")
        c1 = c.shape1.GetBody()
        c2 = c.shape2.GetBody()
