#
# Python includes
#
import math
import random

import logging
log = logging.getLogger('Player')

# 
# Lib includes
#
from main import box2d
from pyglet.window import key

# 
# Game includes
#
from config import *
from item import *       


# The player array
gPlayers = []

forceJump         = float(config.get("PlayerSettings","forceJump"))
forceJumpJump     = float(config.get("PlayerSettings","forceJumpJump"))
speedWalk         = float(config.get("PlayerSettings","speedWalk"))
speedWalkJump     = float(config.get("PlayerSettings","speedWalkJump"))
standingTolerance = float(config.get("PlayerSettings","standingTolerance"))
walkingTolerance  = float(config.get("PlayerSettings","walkingTolerance"))
AIR_JUMP_FRAMES   = float(config.get("PlayerSettings","AIR_JUMP_FRAMES"))
dampingOffset     = 0.05
dampingMultiplier = 0.95

density           = float(config.get("PlayerSettings","density"))
restitution       = float(config.get("PlayerSettings","restitution"))
friction          = float(config.get("PlayerSettings","friction"))

###############################################################################
# Player states.
###############################################################################

#EPlayerState_Idle    = 0
#EPlayerState_Walking = 1
#EPlayerState_Jumping = 2

class Player(SquareItem):
    speed = 0
    def __init__(self, scroller, world, imageBank, isBlue, position=(500,600), rotation=0, scale=1, opacity = 255, color=(255,255,255), anchor = None):
        
        self.imageBank = imageBank
        self.isBlue = isBlue
        self.mSibling = None

        self.mIsStanding   = False
        self.mIsWalking    = False
        self.mWalkingLeft  = False
        self.mWalkingRight = False
        self.mIsHolding    = False
        self.mAirJumpFrame = 0
        
        # Keyboard Map
        self.keyboardMap = {}
        self.keyboardMap['Jump'] = key.W
        self.keyboardMap['Left'] = key.A
        self.keyboardMap['Right'] = key.D
        self.keyboardMap['LetGo'] = key.S

        name = "PlayerBlue"
        if not self.isBlue:
            name = "PlayerRed"
            self.keyboardMap['Jump'] = key.UP
            self.keyboardMap['Left'] = key.LEFT
            self.keyboardMap['Right'] = key.RIGHT
            self.keyboardMap['LetGo'] = key.DOWN


        super( Player, self ).__init__(name, world, self.imageBank["PLAYER_IDLE0"], position, rotation, scale, opacity, color, anchor, True, 
                                       density=density, restitution=restitution, friction=friction)#,  usefg = [ 0x0002,0x0004+0x0008,1])
        
        """
        
        # Add a shape to the Body for feet sensor
        boxDef = box2d.b2PolygonDef()
        
        boxDef.SetAsBox(width, height, box2d.b2Vec2(0,0), 0)
        boxDef.density = 0.2
        boxDef.restitution = 0.16
        boxDef.friction = 0.85
        
        boxDef.filter.categoryBits = fg[0]
        boxDef.filter.maskBits = fg[1]
        boxDef.filter.groupIndex = fg[2]


        
        self.foots = self.body.CreateShape(boxDef)
        """

        self.scroller = scroller

        # Key states.S
        self.keys = {}
        self.keys['Jump'] = False
        self.keys['Left'] = False
        self.keys['Right'] = False
        self.keys['LetGo'] = False

        self.holdJoint = None
        self.commands = []
        self.letMeGo = False
        self.createMeAJoint = None
        self.setAFlag = True

        #self.mStateLast = EPlayerState_Idle
        #self.mStateCurr = EPlayerState_Idle

    def update(self, dt):
        SquareItem.update(self, dt)
        
        if self.mIsWalking:
            velocity = self.body.GetLinearVelocity()
            velocity.x *= dampingMultiplier
            if velocity.x < dampingOffset or velocity.x > -dampingOffset:
                velocity.x = 0
            else:
                velocity -= dampingOffset
            self.body.SetLinearVelocity(velocity)

        if self.keys['Left'] == True:
            self.Left()

        if self.keys['Right'] == True:
            self.Right()
            
        """ 
        if (self.isBlue):
            if config.get("LevelSettings","Scroll")== "True":
                #" ""
                log.info("UpdateScroll")
                pos = self.body.GetPosition()
                pos = [pos.x*gPPM, pos.y*gPPM]
                log.debug("Pos:"+str(pos))
                
                pos = self.parent.phys.world.to_world(pos)
                log.debug("Pos:"+str(pos))
        
                self.scroller.set_focus( pos[0], pos[1])
                #" ""
                
                
                #self.scroller.set_focus(self.x, self.y)
                #self.parent.set_view(0, 0, 1024, 768)
        """        
        for command in self.commands:
           eval(command)
    
        justLanded = False

        #TODO: Use aabb query for standing.
        #shapeNum, shapeList = self.worlf.world.Query(aabb, shapeNumMax)

        for collision in self.mCollisionList:
            if collision[2][1] == -1:
                if not self.mIsStanding:
                    self.mIsStanding = True
                    justLanded = True
                break

        if not justLanded:

            movingUp   = self.body.GetLinearVelocity().y > standingTolerance
            movingDown = self.body.GetLinearVelocity().y < -standingTolerance
            falling    = movingUp or movingDown or not self.mIsStanding

            if falling and self.mIsStanding:
                self.SetAnim("PLAYER_JUMP")
            elif not self.mIsStanding and not falling:
                self.OnLanding()

            self.mIsStanding = not falling

        walkingRight    = self.body.GetLinearVelocity().x > walkingTolerance
        walkingLeft     = self.body.GetLinearVelocity().x < -walkingTolerance
        isStandingStill = self.mIsStanding and not ( walkingLeft or walkingRight )
        isWalking       = self.mIsStanding and ( walkingLeft or walkingRight )

        if isWalking and not self.mIsWalking:
            #Started walking.
            if walkingLeft:
                self.SetAnim("PLAYER_WALK", True)
                self.mWalkingLeft = True
            else:
                self.SetAnim("PLAYER_WALK")
                self.mWalkingLeft = False
        elif not isWalking and self.mIsWalking:
            #Stopped walking.
            self.SetAnim("PLAYER_IDLE0")
        elif isWalking and self.mIsWalking:
            #Still walking, changing direction.
            if self.mWalkingLeft and walkingRight:
                self.SetAnim("PLAYER_WALK")
            elif self.mWalkingRight and walkingLeft:
                self.SetAnim("PLAYER_WALK", True)

        self.mIsWalking      = isWalking
        self.mIsWalkingLeft  = walkingLeft
        self.mIsWalkingRight = walkingRight

        self.mCollisionList = []

        if self.mAirJumpFrame > 0:
            self.mAirJumpFrame -= 1

    ###############################################################################
    # Anim events.
    ###############################################################################

    def on_animation_end(self):
        if self.holdJoint:
            return

        #print "..."   , self.image  
        if (self.image == self.imageBank["PLAYER_IDLE1"]):
            #print "idle1"
            if (random.randint(1, 4) == 3):
                #print "idle changed1"
                self.SetAnim("PLAYER_IDLE0") 
            return

        #print "-->"
        if (self.image == self.imageBank["PLAYER_IDLE0"]):
            #print "idle"
            if (random.randint(0, 10) == 3):
                #print "idle changed"
                self.SetAnim("PLAYER_IDLE1") 
            return    

        if (self.image == self.imageBank["PLAYER_JUMP"]):
            self.SetAnim("PLAYER_LAND")
        elif (self.image == self.imageBank["PLAYER_LAND"]):
            self.SetAnim("PLAYER_IDLE0")    
        elif (self.image == self.imageBank["PLAYER_TAKEOFF"]):
            self.SetAnim("PLAYER_JUMP")

    ###############################################################################
    # Key events.
    ###############################################################################

    def KeyPress(self, pressed):

        if pressed == self.keyboardMap['Jump']:
            self.Jump()
            self.keys['Jump'] = True
        elif pressed == self.keyboardMap['Left']:
            self.Left()
            self.keys['Left'] = True
        elif pressed == self.keyboardMap['Right']:
            self.Right()
            self.keys['Right'] = True
        elif pressed == self.keyboardMap['LetGo']:
            log.debug("pressed == key.DOWN")
            if self.holdJoint:
                log.debug("self.holdJoint"+str(self.holdJoint))
                self.LetGo()
            self.keys['LetGo'] = True

    def KeyRelease(self, released):
        if released == self.keyboardMap['LetGo']:
            self.keys['LetGo'] = False        
            
        if self.holdJoint:
            return
        
        if released == self.keyboardMap['Jump']:
            self.keys['Jump'] = False
        elif released == self.keyboardMap['Left']:
            self.keys['Left'] = False
        elif released == self.keyboardMap['Right']:
            self.keys['Right'] = False

    ###############################################################################
    # Movement.
    ###############################################################################
    
    def DoSwing(self):
        # get siblings position
        # calculate proper verctor (user must be hold ing left or right and press jump)
        # apply.

        if self.mSibling.mIsHolding and not self.mIsHolding:
            pass

        pass

    def DoAirDive(self):

        force = forceDiveJump
        f = self.body.GetWorldVector(box2d.b2Vec2(0.0, -force))
        p = self.body.GetWorldPoint(box2d.b2Vec2(0.0, 0.0))
        self.body.ApplyForce(f, p)

    def DoAirJump(self):
        #if self.mAirJumpFrame > 0:
        #    return

        force = forceJumpJump
        f = self.body.GetWorldVector(box2d.b2Vec2(0.0, force))
        p = self.body.GetWorldPoint(box2d.b2Vec2(0.0, 0.0))
        self.body.ApplyForce(f, p)
        #self.mAirJumpFrame = AIR_JUMP_FRAMES

    def DoStandingJump(self):
        force = forceJump
        f = self.body.GetWorldVector(box2d.b2Vec2(0.0, force))
        p = self.body.GetWorldPoint(box2d.b2Vec2(0.0, 0.0))
        self.body.ApplyImpulse(f, p)

    def Jump(self):    
        if self.holdJoint:
            return
        
        if self.mIsStanding:
            self.DoStandingJump()
        else:
            self.DoAirJump()

    def Left(self):
        if self.holdJoint:
            return

        speed = -speedWalk

        if not self.mIsStanding:
            speed = -speedWalkJump

        velocity = self.body.GetLinearVelocity()
        velocity.x = speed
        self.body.SetLinearVelocity(velocity)

    def Right(self):
        if self.holdJoint:
            return
       
        speed = speedWalk

        if not self.mIsStanding:
            speed = speedWalkJump

        velocity = self.body.GetLinearVelocity()
        velocity.x = speed
        self.body.SetLinearVelocity(velocity)

    def LetGo(self):
        log.debug("LetGo")
        #self.parent.phys.world.world.DestroyJoint(self.holdJoint)
        #self.holdJoint = None
        #self.body.WakeUp()
        self.letMeGo = True
    
    def OnLanding(self):
        self.SetAnim("PLAYER_LAND")
        self.mAirJumpFrame = 0 # Reset air jump frames .. move these to OnLanded etc

    def SetAnim(self, anim, flip=False):
        self.image = self.imageBank[anim].get_transform(flip_x=flip)
        self.image_anchor = (self.image.get_max_width()/2, self.image.get_max_height()/2)

    def Collide(self, obj):
        log.debug("Collide")
        if (config.has_section("ItemSettings"+obj.itemType)):
            for item, value in config.items("ItemSettings"+obj.itemType):
                if value == "True":
                    converted = "self."+item.capitalize()+"(obj)"
                    log.debug(converted)
                    eval(converted)
            return

    def Static(self, obj):
        log.debug("Static")

    def Hold(self, obj):
        log.debug("Hold "+obj.itemType)
        if self.keys['LetGo'] == False:
            self.createMeAJoint = obj

    def Pickup(self, obj):
        log.debug("Pickup "+obj.itemType)
        #obj.taggedForRemoval = True

    def Use(self, obj):
        log.debug("Use "+obj.itemType)
        if (obj.itemType.startswith("DOOR_")):
            if obj.open:
                self.parent.PlayerWin()
                return

            colour = obj.itemType[len("DOOR_"):]
            log.debug("\tDoor: "+colour)
            for item in self.parent.HUD.inventory.keys():
                log.debug("\t\tInv Item: "+item)
                if ( item.endswith(colour)):
                    log.debug("\t\t\t"+ obj.itemType + " : "+item)
                    self.parent.HUD.RemoveFromInventory(item)
                    self.parent.UnlockDoor(obj.itemType)
                    return


    def Autoconsume(self, obj):
        log.debug("AutoConsume "+obj.itemType)
        if obj.itemType == "MUSHROOM" and obj.taggedForRemoval == False:
            obj.taggedForRemoval = True
            self.parent.HUD.AddLife()

    def Inventory(self, obj):
        log.debug("Inventory "+str(obj.itemType))
        if obj.taggedForRemoval == False:
            obj.taggedForRemoval = True
            self.parent.HUD.AddtoInventory(obj.itemType)
